/*
  Meteorino
  © Fabio Marzocca - 2018 - marzoccafabio@gmail.com
  v.2.0.0 - august 2018
  
  - using an external timer TPL5110 to switch off the board
  
*/


#include "ThingSpeak.h"
#include <ESP8266WiFi.h>
#include "DHTesp.h"
#include "user_config.h"


ADC_MODE(ADC_TOUT);

/* NOTE:- All of these values are configured in user_config.h.  */
const byte ip[] = WIFI_IPADDR;
const byte gateway[] = WIFI_GATEWY;
const byte netmask[] = WIFI_NETMSK;
const byte dnssrv[] = WIFI_DNSSRV;

//Thingspeak
unsigned long myChannelNumber = TS_CHANNEL_NR;
const char * myWriteAPIKey = TS_WRITE_KEY;

// IFTTT
const char* IFTTTapiKey = IFTTT_KEY;
const char* IFTTThost = "maker.ifttt.com";

#define DONEPIN 5 //GPIO 5, D1 on NodeMCU

#define DHTPIN 4     // what digital pin we're connected to (D2 on NodeMCU)
#define DHTTYPE DHT22   // DHT 22  (AM2302), AM2321

/*  Globals.  */
int conn_tries = 0;     // WiFi re-connect counter.

WiFiClient  client;
DHTesp dht;

float readBattery() {
  float LSB = 0.0066666667;  //calculated and adjusted per http://esp8266-projects.org/2016/08/esp8266-internal-adc-2-the-easy-way-example/
  int k=0;
  for (int i = 0; i<10; i++){ // take average reading
    k += analogRead(A0);
  }
  k= k/10;
  #ifdef DEBUG
      Serial.println(k);
  #endif
  return (k*LSB);
}


void sendDataToTS() {
   float vdd;

   dht.setup(DHTPIN, DHTesp::DHTTYPE); // Connect DHT sensor to GPIO 4
  //Read Battery level 
  vdd = readBattery();

  if (vdd <= 3.40) {
    SendEventToIFTTT("battery_low");
  }

  float t = dht.getTemperature();
  float h = dht.getHumidity();

    // Check if any reads failed and exit early (to try again).
  if (isnan(h) || isnan(t) ) {
    #ifdef DEBUG
      Serial.println("Failed to read from DHT sensor!");
    #endif
    SendEventToIFTTT("DHT22_fail");
    return;
  }

  float hic = dht.computeHeatIndex(t, h, false);
  float dp = dht.computeDewPoint(t,h,false);
  
  //DHT22 tends to show higher temp. Adjusting
  t = t-0.5;

 
  #ifdef DEBUG
    Serial.print(F("Battery="));
    Serial.println(vdd);
    Serial.print(F("Temp.="));
    Serial.println(t);
    Serial.print(F("Humid="));
    Serial.println(h);
    Serial.print(F("Heat="));
    Serial.println(hic);
    Serial.print(F("Dew Point="));
    Serial.println(dp);
  #endif
  
  // Write to ThingSpeak. There are up to 8 fields in a channel, allowing you to store up to 8 different
  // pieces of information in a channel. 
  ThingSpeak.setField( 1, t );
  ThingSpeak.setField( 2, h );
  ThingSpeak.setField( 3, hic );
  ThingSpeak.setField( 4, vdd );
  ThingSpeak.setField( 5, dp );
   
  int writeSuccess = ThingSpeak.writeFields( myChannelNumber, myWriteAPIKey );
  if (!writeSuccess) {
    SendEventToIFTTT("TS_fail");
  }
}

/* Send an event to IFTT
 *  
*/
void SendEventToIFTTT(char *event) {
  const int httpPort = 80;
  if (!client.connect(IFTTThost, httpPort)) {
    #ifdef DEBUG
     Serial.println("Connection to IFTTT server failed");
    #endif
    return;
  }
  String url = "/trigger/";
  url += event;
  url += "/with/key/";
  url += IFTTTapiKey;
  
  // This will send the request to the server
  client.print(String("GET ") + url + " HTTP/1.1\r\n" +
               "Host: " + IFTTThost + "\r\n" + 
               "Connection: close\r\n\r\n");
  delay (1000);
}


void setup() {
 delay(1500); //stabilize power line

 pinMode(DONEPIN, OUTPUT);
 digitalWrite(DONEPIN, LOW);
 
 #ifdef DEBUG
  Serial.begin(115200);
  delay(200);
  Serial.println(F("\n"));
 #endif

 WiFi.config(IPAddress(ip), IPAddress(gateway),
                IPAddress(netmask), IPAddress(dnssrv));
 WiFi.begin(STA_SSID, STA_PASS, WIFI_CHANNEL, NULL);

 while ((WiFi.status() != WL_CONNECTED) && (conn_tries++ < WIFI_RETRIES)) {
    delay(200);
    #ifdef DEBUG
        Serial.print(F("."));
     #endif
    }

 if (WiFi.status() != WL_CONNECTED) {
  #ifdef DEBUG
        Serial.println(F("No WiFi connection ...powering off for a cycle."));
  #endif
         // toggle DONE so TPL knows to cut power!
        while (1) { 
        digitalWrite(DONEPIN, HIGH); 
        delay(1);
        digitalWrite(DONEPIN, LOW);
        delay(1);
        }
    }
  
 #ifdef DEBUG
    Stat_WiFi();
 #endif

 ThingSpeak.begin(client);
 sendDataToTS();
 #ifdef DEBUG
    Serial.println(F("Done. Switching off for 30 minutes"));
 #endif
 
 // toggle DONE so TPL knows to cut power!
 while (1) { 
  digitalWrite(DONEPIN, HIGH); 
  delay(1);
  digitalWrite(DONEPIN, LOW);
  delay(1);
 }
}


void loop() { 
  //nothing here, as we never reach this.
}

void Stat_WiFi() {
  #ifdef DEBUG
    Serial.println();
    WiFi.printDiag(Serial);
    Serial.print(F("IP: "));
    Serial.println(WiFi.localIP());
    Serial.println();
  #endif
}
